==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/fir/analysis/checkers/expression/FirSpreadOfNullableChecker.kt <==
/*
 * Copyright 2010-2021 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.checkers.expression

import ksp.org.jetbrains.kotlin.KtSourceElement
import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.diagnostics.reportOn
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.diagnostics.FirErrors
import ksp.org.jetbrains.kotlin.fir.expressions.FirExpression
import ksp.org.jetbrains.kotlin.fir.expressions.FirFunctionCall
import ksp.org.jetbrains.kotlin.fir.expressions.FirSpreadArgumentExpression
import ksp.org.jetbrains.kotlin.fir.expressions.FirVarargArgumentsExpression
import ksp.org.jetbrains.kotlin.fir.types.ConeFlexibleType
import ksp.org.jetbrains.kotlin.fir.types.canBeNull
import ksp.org.jetbrains.kotlin.fir.types.resolvedType

object FirSpreadOfNullableChecker : FirFunctionCallChecker(MppCheckerKind.Common) {
    override fun check(expression: FirFunctionCall, context: CheckerContext, reporter: DiagnosticReporter) {
        fun checkAndReport(argument: FirExpression, source: KtSourceElement?) {
            val coneType = argument.resolvedType
            if (argument is FirSpreadArgumentExpression && !argument.isFakeSpread && coneType !is ConeFlexibleType && coneType.canBeNull(context.session)) {
                reporter.reportOn(source, FirErrors.SPREAD_OF_NULLABLE, context)
            }
        }

        for (argument in expression.argumentList.arguments) {
            if (argument is FirVarargArgumentsExpression) {
                for (subArgument in argument.arguments) {
                    checkAndReport(subArgument, argument.source)
                }
            } else {
                checkAndReport(argument, argument.source)
            }
        }
    }
}

==> symbol-processing-aa-embeddable-2.3.3-sources.jar: org/jetbrains/kotlin/fir/analysis/checkers/expression/FirSpreadOfNullableChecker.kt <==
/*
 * Copyright 2010-2021 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.checkers.expression

import ksp.org.jetbrains.kotlin.KtSourceElement
import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.diagnostics.reportOn
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.diagnostics.FirErrors
import ksp.org.jetbrains.kotlin.fir.expressions.FirExpression
import ksp.org.jetbrains.kotlin.fir.expressions.FirFunctionCall
import ksp.org.jetbrains.kotlin.fir.expressions.FirSpreadArgumentExpression
import ksp.org.jetbrains.kotlin.fir.expressions.FirVarargArgumentsExpression
import ksp.org.jetbrains.kotlin.fir.types.ConeFlexibleType
import ksp.org.jetbrains.kotlin.fir.types.canBeNull
import ksp.org.jetbrains.kotlin.fir.types.resolvedType

object FirSpreadOfNullableChecker : FirFunctionCallChecker(MppCheckerKind.Common) {
    context(context: CheckerContext, reporter: DiagnosticReporter)
    override fun check(expression: FirFunctionCall) {
        fun checkAndReport(argument: FirExpression, source: KtSourceElement?) {
            val coneType = argument.resolvedType
            if (argument is FirSpreadArgumentExpression && !argument.isFakeSpread && coneType !is ConeFlexibleType && coneType.canBeNull(context.session)) {
                reporter.reportOn(source, FirErrors.SPREAD_OF_NULLABLE)
            }
        }

        for (argument in expression.argumentList.arguments) {
            if (argument is FirVarargArgumentsExpression) {
                for (subArgument in argument.arguments) {
                    checkAndReport(subArgument, argument.source)
                }
            } else {
                checkAndReport(argument, argument.source)
            }
        }
    }
}
